<?php

namespace AppBundle\Application\Users\UseCases;

use AppBundle\Domain\Exceptions\UserException;
use AppBundle\Domain\Users\User;
use AppBundle\Domain\Users\UserRepository;

/**
 * Class LoginUserUseCase.
 */
class RegisterUserUseCase
{
    /**
     * @var UserRepository
     */
    protected $userRepository;

    /**
     * LoginUserUseCase constructor.
     *
     * @param UserRepository $userRepository
     */
    public function __construct(UserRepository $userRepository)
    {
        $this->userRepository = $userRepository;
    }

    /**
     * @param RegisterUserRequest $request
     *
     * @return RegisterUserResponse
     *
     * @throws UserException
     */
    public function execute(RegisterUserRequest $request)
    {
        $email = $request->email;
        $password = $request->password;
        $repassword = $request->repassword;

        if (false === filter_var($email, FILTER_VALIDATE_EMAIL)) {
            throw new UserException('Invalid Email');
        }

        if ($password !== $repassword) {
            throw new UserException('Password don\'t match');
        }

        /*
         * @var User
         */
        $user = $this->userRepository->userWithCredentials($email, $password);

        if ((null !== $user) && ($user->isValid())) {
            throw new UserException('User exists');
        }

        $user = User::create($email, $password);
        $res = $this->userRepository->register($user);

        if (!$res) {
            throw new UserException('User not create');
        }

        $response = new RegisterUserResponse();
        $response->email = $user->email();
        $response->id = $user->id();

        return $response;
    }
}
